# -*- coding:utf-8 -*-

from DatabaseLibrary.connection_manager import ConnectionManager
from DatabaseLibrary.query import Query
from DatabaseLibrary.assertion import Assertion
from SCLibrary.base import keyword
import json

from robot.api import logger
from robot.libraries.BuiltIn import BuiltIn
from .websql import WebSQL


class DBKeywords(ConnectionManager, Query, Assertion):
    env = 'dev'
    websql = None
    host = ''
    db = ''

    def __init__(self):
        super().__init__()
        built_in = BuiltIn()
        self.env = built_in.get_variable_value("${RF_ENV}", 'dev')
        if self.env == 'prepub' or self.env == 'prod':
            uname = built_in.get_variable_value("${WEBSQL_UNAME}", '')
            pwd = built_in.get_variable_value("${WEBSQL_PWD}", '')
            if uname == '' or pwd == '':
                logger.error('线上环境需要填写WebSQL用户密码: WEBSQL_UNAME, WEBSQL_PWD')
                return
            self.websql = WebSQL(uname, pwd)

    @keyword("DB Connect")
    def db_connect_to_mysql(self, db):
        """ 连接数据库
        ``db`` 字典，例：{'db':'mysql','host':'172.17.40.214','port=7003','name=todo','user':'root','pwd':'root'}
        其中db为连接的数据库类型名称，默认mysql,  name为数据库名
        """
        if self.env == 'prepub' or self.env == 'prod':
            self.host = db['host']
            self.db = db['db']
            return

        module = db['db'] if 'db' in db else 'pymysql'
        if module.upper() == 'MYSQL':
            module = 'pymysql'
        elif module.upper() == 'POSTGRESQL' or module.upper() == 'PGSQL':
            module = 'psycopg2'
        super(DBKeywords, self).connect_to_database(dbapiModuleName=module, dbName=db['name'],
                                                    dbUsername=db['user'], dbPassword=db['pwd'], dbHost=db['host'], dbPort=db['port'], dbCharset='utf8')

    @keyword("DB Disconnect From Database")
    def db_disconnect_db(self):
        """ 断开所有的数据库连接 """
        if self.env == 'prepub' or self.env == 'prod':
            return
        super(DBKeywords, self).disconnect_from_database()

    @keyword("DB Query")
    def db_query(self, selectStatement, sansTran=False, returnAsDict=True):
        """ 连接数据库
        ``selectStatement``  select语句

        ``sansTran`` 是否处于事务

        ``returnAsDict`` 结果作为字典返回，默认True
        """
        if self.env == 'prepub' or self.env == 'prod':
            return self.websql.query(self.host, self.db, selectStatement)
        return super(DBKeywords, self).query(selectStatement, sansTran, returnAsDict)

    @keyword("DB Execute Sql Script")
    def db_execute_sql_script(self, sqlScriptFileName, sansTran=False):
        """ 执行一个SQL脚本文件
        ``sqlScriptFileName``  脚本目录

        ``sansTran`` 是否处于事务
        """
        if self.env == 'prepub' or self.env == 'prod':
            return self.websql.query(self.host, self.db, sqlScriptFileName)
        return super(DBKeywords, self).execute_sql_script(sqlScriptFileName, sansTran)

    @keyword("DB Execute Sql String")
    def db_execute_sql_string(self, sqlString, sansTran=False):
        """ 执行一个SQL
        ``sqlString``  sql

        ``sansTran`` 是否处于事务
        """
        if sqlString.startswith('select') or sqlString.startswith('SELECT'):
            if self.env == 'prepub' or self.env == 'prod':
                return self.websql.query(self.host, self.db, sqlString)
            return self.query(sqlString, sansTran, True)

        if self.env == 'prepub' or self.env == 'prod':
            logger.error('预发／线上环境禁止非读操作!')
            return None
        return super(DBKeywords, self).execute_sql_string(
            sqlString, sansTran)

    @keyword("DB Execute")
    def de_execute(self, sqlString, sansTran=False):
        """ 执行一个SQL
        ``sqlString``  sql

        ``sansTran`` 是否处于事务
        """
        self.db_execute_sql_string(sqlString, sansTran=False)

    @keyword("DB Check If Exists In Database")
    def db_exists_in_db(self, selectStatement, sansTran=False):
        """ 断言查询结果是否存在
        ``selectStatement``  查询语句

        ``sansTran`` 是否处于事务
        """
        if self.env == 'prepub' or self.env == 'prod':
            result = self.db_query(selectStatement)
            if len(result) <= 0:
                raise AssertionError("Expected to have have at least one row from '%s' "
                                 "but got 0 rows." % selectStatement)
            return
        super(DBKeywords, self).check_if_exists_in_database(
            selectStatement, sansTran)

    @keyword("DB Check If Not Exists In Database")
    def db_not_exists_in_db(self, selectStatement, sansTran=False):
        """ 断言查询结果是否不存在 """
        if self.env == 'prepub' or self.env == 'prod':
            result = self.db_query(selectStatement)
            if len(result) > 0:
                raise AssertionError("Expected to have have no rows from '%s' "
                                 "but got some rows : %s." % (selectStatement, result))
            return
        super(DBKeywords, self).check_if_not_exists_in_database(
            selectStatement, sansTran)



import requests
import urllib.parse
import re
import json
import sys
from .crpt import SHA1, bytes2hex, rc4_arr, utf2bytes, bytes2utf, hex2bytes

from robot.api import logger

class WebSQL(object):
  url = 'https://mysqlweb.souche-inc.com/dbninja/'
  url_data = '%sdata.php' % url

  def __init__(self, uname, passwd):
    super().__init__()
    self.key = passwd
    pwd = SHA1(self.key)
    self.session = self.login_for_session(uname, pwd)

  def login_for_session(self, user_name, passwd):
    ''' 登录获取 session '''
    try:
      data = { 'uname': user_name, 'passwd': passwd }
      r_login = requests.post(self.url, data=data)
      return r_login.headers.get('Set-Cookie')
    except Exception as e:
      logger.error(e)
      sys.exit(1)

  def get_dbs(self):
    ''' 获取数据库 '''
    try:
      headers = { 'cookie': self.session }
      request_body = '{"action":"load_config"}'
      r_dbs = requests.post(self.url_data, headers=headers, data=self.encry(request_body))
      text_dbs = self.decry(r_dbs.text)
      json_dbs = json.loads(text_dbs)
      dbs = []
      for host in json_dbs.get('hosts'):
        dbs.append({'cid': host['cid'], 'host' : host['label']})
      return dbs
    except Exception as e:
      logger.error(e)
      sys.exit(1)

  def find_db(self, host_label, db):
    dbs = self.get_dbs()
    for item in dbs:
      if item.get('host') == host_label:
        return item.get('cid')
    return None

  def query(self, host_label, db, sql):
    # 查询目标数据库的cid
    cid = self.find_db(host_label, db)
    if cid == None:
      logger.error("WebSQL中无法找到该数据库信息")
      sys.exit(1)

    # 根据sql查询fields, pageCount, outFile
    headers = { 'cookie': '%s;sql_commandEditor=%s' % (self.session, urllib.parse.quote(sql))}
    request_body ='{"action":"query","cid":"%s","db":"%s","query":"%s","params":[],"flags":5,"toFile":false,"limited":true}' % (cid, db, sql)
    r_outfile = requests.post(self.url_data, headers = headers, data = self.encry(request_body))
    text = self.decry(r_outfile.text)
    result = json.loads(text)
    fields, page_count, out_file = self.parse_fields(result)
    if page_count > 10:
      logger.warn("查询结果大于1000, 返回1000个结果（一共%s页）" % page_count)
    # 根据outfile查询结果
    return self.poly_result(headers, fields, page_count, out_file, 0, [])

  def poly_result(self, headers, fields, page_count, out_file, index, data):
    request_body = '{"action":"get_query_result","file":"%s","result":0,"page":%d}' % (out_file, index)
    r_result = requests.post(self.url_data, headers = headers, data = self.encry(request_body))
    text = self.decry(r_result.text)
    text = text.replace(',\n];', ']')
    text = text.replace('\n];', ']')
    result = json.loads(text)
    for item in result:
      data.append(item)
    if (index + 1) < page_count:
      return self.poly_result(headers, fields, page_count, out_file, index + 1, data)
    else:
      return self.format_result(fields, data)
  
  def encry(self, data):
    return '!:' + bytes2hex(rc4_arr(self.key, utf2bytes(data)))

  def decry(self, data):
    if data[0:2] == "!:":
      text = bytes2utf(rc4_arr(self.key, hex2bytes(data[2:])))
      self.key = re.findall(r'^\!\:(\w+)\n', text)[0]
      text = text[2+len(self.key):]
      return text
    logger.error('返回异常，不包含 !: 加密数据')
    sys.exit(1)

  def parse_fields(self, result):
    data = result.get('results')[0]
    fields = data.get('fields')
    db_fields = []
    for f in fields:
      if f.get('type') == 'number':
        db_fields.append(f.get('fieldAlias'))
      else:
        db_fields.append(f.get('field'))
    page_count = data.get('pageCount')
    out_file = result.get('outFile')
    return db_fields, page_count, out_file

  def format_result(self, fields, result):
    array = []
    for i in range(len(result)):
      item = dict()
      for j in range(len(fields)):
        item[fields[j]] = result[i][j]
      array.append(item)
    return array